# This file was auto-generated by Fern from our API Definition.

import typing

MapValue = typing.Union[
    typing.Optional[str],
    typing.Optional[int],
    typing.Optional[bool],
    typing.Optional[typing.List[str]],
]
