# This file was auto-generated by Fern from our API Definition.

import typing

from ...commons.types.usage import Usage
from .open_ai_usage import OpenAiUsage

IngestionUsage = typing.Union[Usage, OpenAiUsage]
