# This file was auto-generated by Fern from our API Definition.

from .create_score_config_request import CreateScoreConfigRequest
from .score_configs import ScoreConfigs

__all__ = ["CreateScoreConfigRequest", "ScoreConfigs"]
